#include <math.h>
#include "common.h"
#include "INA226.h"
float currentLSB ;

// 初始化 INA226
bool INA226_begin(int sda, int scl) {
    pinMode(sda, OUTPUT);
    pinMode(scl, OUTPUT);
    digitalWrite(sda, HIGH);
    digitalWrite(scl, HIGH);
    return true;
}

// 配置 INA226
bool INA226_configure(int sda, int scl) {
    uint16_t config = 0x4127; // 默认配置：16 次平均，1.1ms 转换时间，连续模式
    i2c_start(sda, scl);
    i2c_write_byte(sda, scl, INA226_ADDRESS << 1);
    i2c_write_byte(sda, scl, INA226_REG_CONFIG);
    i2c_write_byte(sda, scl, config >> 8);
    i2c_write_byte(sda, scl, config & 0xFF);
    i2c_stop(sda, scl);
    return true;
}

// 校准 INA226
bool INA226_calibrate(int sda, int scl, float rShuntValue, float iMaxExpected) {
    
     currentLSB = iMaxExpected/32767.0 ; // 计算电流 LSB,/INA226_REG_CALIBRATION

    uint16_t calibrationValue = (uint16_t)(0.00512 / (currentLSB * rShuntValue));

    i2c_start(sda, scl);
    i2c_write_byte(sda, scl, INA226_ADDRESS << 1);
    i2c_write_byte(sda, scl, INA226_REG_CALIBRATION);
    i2c_write_byte(sda, scl, calibrationValue >> 8);
    i2c_write_byte(sda, scl, calibrationValue & 0xFF);
    i2c_stop(sda, scl);
    return true;
}

// 读取总线电压
float INA226_readBusVoltage(int sda, int scl) {
    i2c_start(sda, scl);
    i2c_write_byte(sda, scl, INA226_ADDRESS << 1);
    i2c_write_byte(sda, scl, INA226_REG_BUSVOLTAGE);
    i2c_start(sda, scl);
    i2c_write_byte(sda, scl, (INA226_ADDRESS << 1) | 1);
    uint8_t high = i2c_read_byte(sda, scl, true);
    uint8_t low = i2c_read_byte(sda, scl, false);
    i2c_stop(sda, scl);
    int16_t value = (high << 8) | low;
    return value * 0.00125; // 1.25mV/LSB
}

// 读取电流
float INA226_readCurrent(int sda, int scl) {
    i2c_start(sda, scl);
    i2c_write_byte(sda, scl, INA226_ADDRESS << 1);
    i2c_write_byte(sda, scl, INA226_REG_CURRENT);
    i2c_start(sda, scl);
    i2c_write_byte(sda, scl, (INA226_ADDRESS << 1) | 1);
    uint8_t high = i2c_read_byte(sda, scl, true);
    uint8_t low = i2c_read_byte(sda, scl, false);
    i2c_stop(sda, scl);
    int16_t value = (high << 8) | low;
    return value * currentLSB; // 假设电流 LSB 为 1mA
}